// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - HUD
// Contains functions related to drawing HUD elements
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

#define CIRCLE_PADDING			120.0

// Contains information about numbers, etc. that will be drawn on the screen
struct ZT2HUDInfo
{
	int TimeLeft;					// Actual time left
	int ZombiesLeft;				// Zombies we NEED to kill
};

// ------------------------/
// SERVER -> CLIENT
// Create the non-persistent HUD
// ------------------------/

script "RequestHUDCreation" CLIENTSIDE
{
	if (WrongClient())
		terminate;
		
	ACS_NamedExecuteAlways("CreateZT2HUD", 0, false);
}

// ------------------------/
// Mainline hook for ZT2 HUD
// ------------------------/

script "CreateZT2HUD" (int persistent)
{
	// Persistent elements are things that ALWAYS show
	// Spectators will also see these!
	
	if (persistent)
	{
		// Top-right circle
		ACS_NamedExecute("HUD_BioCircleController",0,0,0,0);
	}
	
	// For REAL players only
	else
	{
		ACS_NamedExecute("HUD_WaveCountdownController",0,0,0,0);
	}
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Controls the circle in the top-right
// ------------------------/

script "HUD_BioCircleController"
{
	// The HUD elements need to be re-drawn
	bool dirty = true;
	
	int lastNumber, lastGoal, lastState;
	bool traderInProgress;
	bool shouldFormat;
	
	while (true)
	{
		// THE ACTUAL GAME STATE!
		if (lastState != GameState.State)
		{
			dirty = true;
			lastState = GameState.State;
		}
			
		// Actual numbers
		int checkNumber = (GameState.State == STATE_WAVE) ? GameState.HUD.ZombiesLeft : GameState.HUD.TimeLeft;
		if (lastNumber != checkNumber)
		{
			lastNumber = checkNumber;
			dirty = true;
			
			shouldFormat = (GameState.State == STATE_TRADER);
		}
		
		// Trader state
		if (GameState.State == STATE_TRADER && !traderInProgress)
		{
			traderInProgress = true;
			dirty = true;
		}
		else if (GameState.State != STATE_TRADER && traderInProgress)
		{
			traderInProgress = false;
			dirty = true;
		}
		
		if (dirty)
		{
			dirty = false;
			
			// Should we even show the timer?
			if (GameState.State != STATE_LOBBY && GameState.State != STATE_COUNTDOWN)
			{
				// (fixed xPos, fixed yPos, str text, int alX, int alY, int id, fixed sTime = 3.0, fixed fTime = 0.0)
				SetFont(traderInProgress ? "BIOTIME1" : "BIOTIME2");
				HUDMessageUnscaled(-CIRCLE_PADDING, CIRCLE_PADDING, "a", ALIGN_CENTER, ALIGN_CENTER, HID_CIRCLE_BG, 0.0, 0.0);
				
				str numToShow = shouldFormat ? TimeString(checkNumber) : StrParam(d:checkNumber);
				
				SetFont(GameState.State == STATE_TRADER ? "BIGFONT" : "HUGEFONT");
				HUDMessageUnscaled(-CIRCLE_PADDING, CIRCLE_PADDING, numToShow, ALIGN_CENTER, ALIGN_CENTER, HID_CIRCLE_A, 0.0, 0.0);
				
				// Bottom text (FUNNY MEME)
				str botText = StrParam(d:GameState.Wave+1, s:" / ", d:MAX_WAVES);
				SetFont("TYPE24");
				HUDMessageUnscaled(-CIRCLE_PADDING, CIRCLE_PADDING + 40.0, botText, ALIGN_CENTER, ALIGN_CENTER, HID_CIRCLE_B, 0.0, 0.0);
			}
			else
			{
				ClearHUDMessage(HID_CIRCLE_BG);
				ClearHUDMessage(HID_CIRCLE_A);
				ClearHUDMessage(HID_CIRCLE_B);
			}
		}
		
		delay(1);
	}
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Controls wave countdown
// ------------------------/

script "HUD_WaveCountdownController"
{
	// Redraw HUD elements?
	bool dirty;
	bool wasCountdown;
	int lastTime;
	bool didBlood;
	
	ZTPrint("HUD_WaveCountdownController");
	
	while (true)
	{
		// Sync state
		if (GameState.State == STATE_COUNTDOWN && !wasCountdown)
		{
			wasCountdown = true;
			dirty = true;
		}
		else if (GameState.State != STATE_COUNTDOWN && wasCountdown)
		{
			wasCountdown = false;
			dirty = true;
		}
		
		// Sync time
		if (lastTime != GameState.WaveTimer)
		{
			lastTime = GameState.WaveTimer;
			dirty = true;
		}
		
		if (dirty)
		{
			// Show something!
			if (wasCountdown)
			{
				// (fixed xPos, fixed yPos, str text, int alX, int alY, int id, fixed sTime = 3.0, fixed fTime = 0.0)
				str secText = GameState.WaveTimer == 1 ? " SECOND..." : " SECONDS...";
				str showText = StrParam(s:"WAVE ", d:GameState.Wave+1, s:" IS STARTING IN ", d:GameState.WaveTimer, s:secText);
				
				if (GameState.WaveTimer <= 3 && GameState.WaveTimer >= 1)
					LocalAmbientSound("tick/tock", 127);
				
				// Top text
				SetFont("TFNDESC");
				HUDMessageUnscaled(CenterX() * 1.0, -48.0, showText, ALIGN_CENTER, ALIGN_CENTER, HID_CDOWN_TEXT, 0.0, 0.0);
				
				// BLOOD
				if (!didBlood)
				{
					ACS_NamedExecute("HUD_WaveCountdownBlood", 0);
					didBlood = true;
				}
			}
			
			// Hide the countdown messages
			else 
			{
				didBlood = false;
				ClearHUDMessage(HID_CDOWN_TEXT);
				ClearHUDMessage(HID_CDOWN_BLOOD);
			}
			
			dirty = false;
		}
		
		Delay(1);
	}
}

// ------------------------/
// Blood for countdown
// ------------------------/

script "HUD_WaveCountdownBlood"
{
	for (int l=0; l<5; l++)
	{
		LocalAmbientSound("hud/bloodsplat", 127);
		SetFont(StrParam(s:"HUDBLUD", d:l+1));
		HUDMessageUnscaled(CenterX() * 1.0, -48.0, "a", ALIGN_CENTER, ALIGN_CENTER, HID_CDOWN_BLOOD, 0.0, 0.0);
		delay(10);
	}
}

// ------------------------/
// FIGHT!!!
// ------------------------/

void ShowFightImage()
{
	int yPos = FixedMul(ResolutionY()*1.0, 0.6);
	SetFont("FIGHTHUD");
	HUDMessageUnscaled(CenterX() * 1.0, yPos, "a", ALIGN_CENTER, ALIGN_CENTER, HID_FIGHT, 3.0, 1.0);
}