// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - UI
// Contains UI functions
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

#define AV_CENTER 			0.4
#define AV_CENTER_Y 		0.0
#define AV_LEFT 			0.1
#define AV_RIGHT 			0.2

#define X_CENTER			1.5
#define Y_CENTER			0.5

// Alignment for text and such
enum Alignment
{
	ALIGN_LEFT = 0,
	ALIGN_TOP = 0,
	ALIGN_CENTER = 1,
	ALIGN_RIGHT = 2,
	ALIGN_BOTTOM = 2,
};

// Position + Alignment = Final Pos
int AliX(int pos, int al) { return pos + ResolveAlignment(al, false); }
int AliY(int pos, int al) { return pos + ResolveAlignment(al, true); }

// Get the number to ADD based on the alignment value
int ResolveAlignment(int num, bool vertical = false)
{
	switch (num)
	{
		// Left / Top
		case ALIGN_LEFT:
			return AV_LEFT;
		break;
		
		// Center
		case ALIGN_CENTER:
			return vertical ? AV_CENTER_Y : AV_CENTER;
		break;
		
		// Right / Bottom
		case ALIGN_RIGHT:
			return AV_RIGHT;
		break;
	}
	
	return 0.0;
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Half-res of screen
int CenterX()
{
	return ResolutionX() / 2;
}

// Obtain X resolution
int ResolutionX()
{
	if (IsServer())
		return PlayerState[ PlayerNumber() ].ResX;
		
	// Use the cached value
	return GameState.LastResX;
}

// Obtain Y resolution
int ResolutionY()
{
	if (IsServer())
		return PlayerState[ PlayerNumber() ].ResY;
		
	// Use the cached value
	return GameState.LastResY;
}

// ------------------------/
// Begin syncing resolution-related things
// ------------------------/

void StartResolutionHook()
{
	// Resolution cannot be grabbed on dedicated servers
	if (IsServer())
		return;
		
	// Begin our resolution handler script
	ACS_NamedExecute("ResolutionHandler",0,0,0,0);
}

// ------------------------/
// ResolutionHandler
// Sends our res to the server if necessary
// ------------------------/

script "ResolutionHandler" CLIENTSIDE
{
	while (true)
	{
		int ResX = GetScreenWidth();
		int ResY = GetScreenHeight();
		
		// They're different! Let the server know
		if (ResX != GameState.LastResX || ResY != GameState.LastResY)
		{
			// Update them locally
			GameState.LastResX = ResX;
			GameState.LastResY = ResY;
			
			// RELIABLY send them to the server
			ACS_NamedExecute("ResolutionSender",0,ResX,ResY,0);
		}
		
		delay(RES_CHECK_INTERVAL);
	}
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Clears a HUD message
// ------------------------/

void ClearHUDMessage(int id)
{
	HudMessage(s:""; HUDMSG_PLAIN, id, CR_UNTRANSLATED, 0.0, 0.0, 0.2);
}

// ------------------------/
// HUDMessageUnscaled
// Shows an UNSCALED message on the screen
//
// xPos, yPos: 			X and Y coordinates (as fixed)
// text: 				The text to show
// alX, alY:			X and Y alignment values
// id:					ID to use for it
// sTime, fTime:		Show / Fade times
// ------------------------/

void HUDMessageUnscaled(fixed xPos, fixed yPos, str text, int alX, int alY, int id, fixed sTime = 3.0, fixed fTime = 0.0)
{
	int resX, resY, finalX, finalY, type, alignmentX, alignmentY;
	
	// Resolve alignment based on our enum
	alignmentX = ResolveAlignment(alX, false);
	alignmentY = ResolveAlignment(alY, true);
	
	// Convert to floats
	xPos = ToFloat(xPos);
	yPos = ToFloat(yPos);
	
	// Decide our screen resolution
	resX = ResolutionX() * 1.0;
	resY = ResolutionY() * 1.0;
	
	// Wrap them around if they're negative
	xPos = (xPos < 0.0) ? resX + xPos : xPos;
	yPos = (yPos < 0.0) ? resY + yPos : yPos;

	// If showTime is 0.0, it should fade out
	type = (sTime > 0.0) ? (HUDMSG_PLAIN | HUDMSG_FADEOUT) : HUDMSG_PLAIN;

	SetHUDSize(ShaveInt(resX), ShaveInt(resY), false);
	HudMessage(s:text; type, id, CR_UNTRANSLATED, xPos + alignmentX, yPos + alignmentY, sTime, fTime);
	SetHUDSize(0, 0, false);
}