// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - HEADSHOTS
// Contains headshot-related functions
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

int bloodPlayerTID;

// Called from blood when it spawns
script "Script_Blood" (int bloodZ) // blood's Z is already passed automatically
{
	int OwnerZ;

	// Set activator to blood's owner, and retrieve its Z coord
	// This is the monster you're shooting
	SetActivatorToTarget(0);

	// Not a monster!
	if (ClassifyActor(0) & !ACTOR_MONSTER)
		terminate;
	
	// We've already lost our head
	if (CheckInventory("Decapitated"))
		terminate;
		
	// Find our index in the list
	int ZI = GetZombieIndex();
	if (ZI < 0)
		terminate;
		
	// This zombie cannot be headshotted
	if (ZombieList[ZI].Headless)
		terminate;
		
	// Get the zombie's height, regardless of what it's currently at
	// This allows us to headshot monsters that are in the process if dying
	
	int aHeight = ZombieList[ZI].Height;

	int aHealth = GetActorProperty(0, APROP_HEALTH);
	OwnerZ = GetActorZ(0) >> 16;

	// How big is our head?
	int HeadHeight = ZombieList[ZI].HeadHeight;

	// Minimum height for the headshot
	int HSHeightMin = aHeight - HeadHeight;

	// Temporarily change the monster's TID to a new one
	int OID = ActivatorTID();
	int UID = UniqueTID();
	Thing_ChangeTID(0, UID);

	// How much damage should it do?
	int HSDamage = GetHeadshotDamage();

	// PrintBold(s:"\cD- HEADSHOT for ", s:GetActorClass(0), s:"! -\nZ: ", d:bloodZ, s:" / ", d:(OwnerZ + HSHeightMin), 
	// s:"\n", d:HSDamage, s:" DAMAGE\n", s:"HEALTH: ", d:aHealth, s:" -> ", d:aHealth-HSDamage);

	// Wasn't above the height it needed to be, so revert TID
	if (bloodZ < OwnerZ + HSHeightMin)
	{
		Thing_ChangeTID(UID, OID);
		terminate;
	}
		
	ActivatorSound("skull/hit", 127);

	// Play the sound for the player, so they KNOW they scored a headshot
	if (bloodPlayerTID > 0)
	{
		SetActivator(bloodPlayerTID);
		LocalAmbientSound("skull/hit", 127);
		
		// We do damage here, so the player is the instigator of it
		// PrintBold(s:"DAMAGING ", d:UID, s:" WITH ", d:HSDamage, s:" DAMAGE");
		Thing_Damage2(UID, HSDamage, "head");
	}
	
	// Can we set the activator back to the monster?
	if (SetActivator(UID))
	{
		int newHealth = GetActorProperty(0, APROP_Health);
		
		// Revert our TID back to what it was before
		Thing_ChangeTID(UID, OID);

		// Would this kill us?
		// Don't ACTUALLY damage, this sets target
		if (newHealth <= 0)
		{
			GiveInventory("Decapitated", 1);
			SetActorState(0, "Decapitated", true);
		}
	}
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// How much damage should a headshot do?

int GetHeadshotDamage(void) 
{ 
	int HSD = HEADSHOT_DAMAGE;
	
	// At this point, the zed have a unique ID from the last function
	int OldID = ActivatorTID();
	
	// Set activator to the player we're targeting / chasing
	SetActivatorToTarget(0);
	
	bloodPlayerTID = 0;
	
	// Make sure this is actually a player
	// If it is, then store our TID in the global var
	
	if (ClassifyActor(0) & ACTOR_PLAYER)
		bloodPlayerTID = ActivatorTID();
	
	// Set activator back to monster
	SetActivator(OldID);

	return HSD;
}