// -- Utility functions

// Ensure that we have X amount of an inventory item
void InvEnsure(str item, int amt)
{
	int cnt = CheckInventory(item);
	if (amt > cnt)
		GiveInventory(item, amt-cnt);
	else if (cnt > amt)
		TakeInventory(item, cnt - amt);
}

// Check if a button was pressed ONCE, not held
bool ButtonPressed(int input, int BT = -1, int OBT = -1)
{
	if (BT == -1)
		BT = GetPlayerInput(-1, INPUT_BUTTONS);
	if (OBT == -1)
		OBT = GetPlayerInput(-1, INPUT_OLDBUTTONS);
		
	if( (BT & input) && !(OBT & input) )
		return true;
		
	return false;
}

// Create a "fake" hash
// This turns a string into a unique int based on characters
// Fake hashes should be the SAME on client and server
int FakeHash(str hasher)
{
	int hash = 0;
	for (int l=0; l<StrLen(hasher); l++)
		hash += GetChar(hasher, l) + (3*l);
		
	return hash;
}

// Smallest of 2 values
int min (int a, int b)
{
	if (a < b)
		return a;

	return b;
}

// Largest of 2 values
int max (int a, int b)
{
	if (a > b)
		return a;

	return b;
}

// We're playing alone
bool IsOffline()
{
	return !GameState.Online;
}

// Unreal-like AUTHORITY check
bool IsAuthority()
{
	return (GameState.Singleplayer || !GameState.Client);
}

// ONLY on the server! For network games only
bool IsServer()
{
	return (!GameState.Client && GameState.Online);
}

// ONLY on the client! For network games only
bool IsClient()
{
	return (GameState.Client && GameState.Online);
}

// Ensure we're the correct client, for S->C scripts
bool WrongClient()
{
	return (PlayerNumber() != ConsolePlayerNumber());
}

// Quick and lazy proximity check
bool ActorInRange(int XV, int YV, int ZV, int tid2, int range)
{
	int x, y, z;
	int xd, yd, zd;
	
	x = GetActorX(tid2);
	y = GetActorY(tid2);
	z = GetActorZ(tid2);
	
	xd = abs(XV - x);
	if (xd <= range)
		return true;
		
	yd = abs(YV - y);
	if (yd <= range)
		return true;
		
	return false;
}

// Lazy distance between two TID's, AS AN INTEGER
int lazyDistance (int tida, int tidb)
{
	int xa, ya, xb, yb;
	
	xa = ShaveInt(GetActorX(tida));
	ya = ShaveInt(GetActorY(tida));
	
	xb = ShaveInt(GetActorX(tidb));
	yb = ShaveInt(GetActorY(tidb));
	
	int xDist = (xa + xb) / 2;
	int yDist = (xa + xb) / 2;
	
	return (xDist + yDist) / 2;
}

// Distance betwene a point and a TID
int distancePoint (int XV, int YV, int ZV, int tid2)
{
	int x, y, z, d;
	x = XV - GetActorX(tid2) >> 16; // Convert fixed point to integer
	y = YV - GetActorY(tid2) >> 16;
	z = ZV - GetActorZ(tid2) >> 16;
	d = sqrt( x*x + y*y + z*z );
	return d;
}

// Slightly slower version of distancePoint, but has no sqrt looping apparently
int fdistancePoint (int XV, int YV, int ZV, int tid2)
{
	int len;
	int y = YV - getactory(tid2);
	int x = XV - getactorx(tid2);
	int z = ZV - getactorz(tid2);

	int ang = vectorangle(x,y);
	if(((ang+0.125)%0.5) > 0.25) len = fixeddiv(y, sin(ang));
	else len = fixeddiv(x, cos(ang));

	ang = vectorangle(len, z);
	if(((ang+0.125)%0.5) > 0.25) len = fixeddiv(z, sin(ang));
	else len = fixeddiv(len, cos(ang));

	return len >> 16;
}

// Distance between two TIDs
int distance (int tid1, int tid2)
{
	int x, y, z, d;
	x = GetActorX(tid1) - GetActorX(tid2) >> 16; // Convert fixed point to integer
	y = GetActorY(tid1) - GetActorY(tid2) >> 16;
	z = GetActorZ(tid1) - GetActorZ(tid2) >> 16;
	d = sqrt( x*x + y*y + z*z );
	return d;
}

int abs(int x) { return (x < 0) ? -x : x; }

// Cast to integer
int ShaveInt(int num) { return (num>>16); }

// Cast to float, with fractional removed
int ToFloat(int num) { return (num>>16)<<16; }

// Check if a player is alive, in the game, etc.
bool PlayerValid(int PN)
{
	return (PN == 0);
}

// Obtain player TID based on number
int PlayerTID(int num)
{
	return BASE_PLAYER_TID + num;
}

// Lists all ALIVE or DEAD players that need to be ready for the game to start
int DesiredReadyPlayers()
{
	int count = 0;
	
	for (int i=0; i<MAXPLAYERS; i++)
	{
		if (PlayerInGame(i) || PlayerIsSpectator(i) == 2)
			count ++;
	}

	return count;
}

// Pad a string to a certain length
str StrPad(str string, str padder, int len)
{
	while (StrLen(string) < len)
		string = StrParam(s:padder, s:string);
		
	return string;
}

// Return a padded time string from a number
str TimeString(int sec)
{
	// Split the trader text into minutes
	int minutes = ShaveInt(FixedDiv(sec*1.0,60.0));
	str minuteText = StrPad( StrParam(d:minutes), "0", 2 );
		
	// Split the trader text into seconds
	int seconds = sec - (60*minutes);
	str secondText = StrPad( StrParam(d:seconds), "0", 2 );

	return StrParam(s:minuteText,s:":",s:secondText);
}